# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Contract.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='Contract.proto',
  package='communication',
  syntax='proto2',
  serialized_options=b'\n\024de.dlr.fame.protobufB\tContracts',
  serialized_pb=b'\n\x0e\x43ontract.proto\x12\rcommunication\"\xb3\x01\n\rProtoContract\x12\x10\n\x08senderId\x18\x01 \x02(\x03\x12\x12\n\nreceiverId\x18\x02 \x02(\x03\x12\x13\n\x0bproductName\x18\x03 \x02(\t\x12\x19\n\x11\x66irstDeliveryTime\x18\x04 \x02(\x03\x12\x1f\n\x17\x64\x65liveryIntervalInSteps\x18\x05 \x02(\x03\x12+\n\x0e\x65xpirationTime\x18\x06 \x01(\x03:\x13\x39\x32\x32\x33\x33\x37\x32\x30\x33\x36\x38\x35\x34\x37\x37\x35\x38\x30\x37\x42!\n\x14\x64\x65.dlr.fame.protobufB\tContracts'
)




_PROTOCONTRACT = _descriptor.Descriptor(
  name='ProtoContract',
  full_name='communication.ProtoContract',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='senderId', full_name='communication.ProtoContract.senderId', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiverId', full_name='communication.ProtoContract.receiverId', index=1,
      number=2, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='productName', full_name='communication.ProtoContract.productName', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firstDeliveryTime', full_name='communication.ProtoContract.firstDeliveryTime', index=3,
      number=4, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deliveryIntervalInSteps', full_name='communication.ProtoContract.deliveryIntervalInSteps', index=4,
      number=5, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expirationTime', full_name='communication.ProtoContract.expirationTime', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=True, default_value=9223372036854775807,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=34,
  serialized_end=213,
)

DESCRIPTOR.message_types_by_name['ProtoContract'] = _PROTOCONTRACT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoContract = _reflection.GeneratedProtocolMessageType('ProtoContract', (_message.Message,), {
  'DESCRIPTOR' : _PROTOCONTRACT,
  '__module__' : 'Contract_pb2'
  # @@protoc_insertion_point(class_scope:communication.ProtoContract)
  })
_sym_db.RegisterMessage(ProtoContract)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import sys

from famepy.scripts.make_config import run as make_config
from famepy.scripts.convert_results import run as convert_results


def makeFameRunConfig():
	if len(sys.argv) != 2:
		print("Usage: makeFameRunConfig <path/to/configuration/file.yaml>")
	else:
		make_config(sys.argv[1])


def convertFameResults():
	if len(sys.argv) > 2:
		convert_results(sys.argv[1], sys.argv[2].upper())
	elif len(sys.argv) == 2:
		convert_results(sys.argv[1])
	else:
		print("Usage: convertFameResults <path/to/result/file.pb> ['list', 'of', 'agents']")



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import sys
import pathlib as pt
from fnmatch import fnmatch
import os

import yaml

from famepy.source.FieldValidator import FieldValidator, FieldType
from famepy.source.TimeSeriesManager import TimeSeriesManager
from famepy.source.tools import load_yaml, log_and_raise, set_logfile
from famepy.source.FameTime import FameTime
from famepy.protobuf_definitions import InputFile_pb2

DISABLING_YAML_FILE_PREFIX = "DISABLED_"


def add_contracts_from_folder(contracts, path):
    """
    Reads YAML files under the given path and adds their `Contracts` to the given list of contracts.
    Ignores files starting with a disabling prefix
    """
    for contract_file in get_contract_file_list(path):
        with open(pt.Path(path, contract_file), "r") as config_file:
            yaml_data = yaml.load(config_file, Loader=yaml.FullLoader)
            for contract in yaml_data['Contracts']:
                contracts.append(contract)


def get_contract_file_list(path):
    """Returns list of files with ending ".yaml" or ".yml" in given folder not having the disabling prefix"""
    ignore_filter = DISABLING_YAML_FILE_PREFIX + "*"
    file_list = []
    for file_name in os.listdir(path):
        if (fnmatch(file_name, "*.yaml") or fnmatch(file_name, "*.yml")) and not fnmatch(file_name, ignore_filter):
            file_list.append(file_name)
    return file_list


def set_general_properties(properties, proto):
    """Set the general simulation properties in the given proto_buffer"""
    for property_name, property_value in properties.items():
        if not hasattr(property_value, "keys"):
            setattr(proto, property_name, property_value)
        else:
            parent = getattr(proto, property_name)
            for child_property_name, child_property_value in property_value.items():
                if child_property_name == "startTime" or child_property_name == "stopTime":
                    child_property_value = int(FameTime.convert_time_string_to_fame_time_step(child_property_value))
                setattr(parent, child_property_name, child_property_value)


def update_field_values(pb_agent, fields, time_series_manager, field_validator):
    """Adds all fields in the given list to the given agent proto buffer"""
    if fields is not None:
        agent_type = pb_agent.className
        for field_name, field_value in fields.items():
            pb_field = pb_agent.field.add()
            pb_field.fieldName = field_name
            field_type = field_validator.get_field_type(agent_type, field_name)
            if field_validator.is_valid(agent_type, field_name, field_value):
                if field_type is FieldType.INTEGER:
                    pb_field.intValue = field_value
                elif field_type is FieldType.DOUBLE:
                    pb_field.doubleValue.extend([field_value])
                elif field_type is FieldType.ENUM:
                    pb_field.stringValue = field_value
                elif field_type is FieldType.TIME_SERIES:
                    if isinstance(field_value, str):
                        file_name = pt.Path(field_value).as_posix()
                        pb_field.seriesId = time_series_manager.save_get_time_series_id(file_name)
                    else:
                        pb_field.seriesId = time_series_manager.save_get_time_series_id(field_value)
                elif field_type is FieldType.DOUBLE_LIST:
                    for element in field_value:
                        pb_field.doubleValue.extend([element])
                else:
                    log_and_raise("FieldType '{}' not implemented.".format(field_type))
            else:
                log_and_raise("'{}' not allowed in field '{}' of agent {}".format(field_value, field_name, pb_agent.id))


def get_or_error(agent, param):
    """Gets given `param` from dictionary or raises error if field is missing"""
    try:
        return agent[param]
    except KeyError:
        log_and_raise("Cannot find '{}' in `agent` {}".format(param, agent))


def set_agents_and_time_series(agent_list, proto_buffer, field_validator):
    """
    Iterates through all agents, adds them and all of their fields to the given proto buffer and also
    adds all referenced files as time series to the proto_buffer. Ensures proper field parameterization and format.
    """
    time_series_manager = TimeSeriesManager()
    for agent in agent_list:
        agent = convert_keys_to_lower(agent)
        pb_agent = proto_buffer.agent.add()
        pb_agent.className = get_or_error(agent, "type")
        pb_agent.id = get_or_error(agent, "id")
        if "fields" in agent:
            fields = agent.get("fields")
            update_field_values(pb_agent, fields, time_series_manager, field_validator)
    time_series_manager.add_time_series_to_proto_buffer(proto_buffer)


def convert_keys_to_lower(agent):
    """Returns given dictionary with `keys` in lower case"""
    return {keys.lower(): value for keys, value in agent.items()}


def set_contracts(contracts, proto_buffer):
    """Adds all contracts in the given list to the given proto buffer"""
    for contract in contracts:
        pb_contract = proto_buffer.contract.add()
        for key, value in contract.items():
            setattr(pb_contract, key, value)


def run(yaml_file):
    """Executes the main workflow for the building of a FAME configuration file"""
    config_data = load_yaml(yaml_file)
    run_config = config_data["RunConfig"]
    set_logfile(run_config["LogFile"])
    validator = FieldValidator(run_config["Schema"])
    proto_input_data = InputFile_pb2.InputData()
    set_general_properties(config_data["GeneralProperties"], proto_input_data)
    set_agents_and_time_series(config_data["Agents"], proto_input_data, validator)

    contract_list = [] if config_data.get("Contracts") is None else config_data.get("Contracts")
    if "ContractsPath" in run_config:
        contract_folder_path = run_config["ContractsPath"]
        add_contracts_from_folder(contract_list, contract_folder_path)
    set_contracts(contract_list, proto_input_data)

    f = open(run_config["OutputFile"], "wb")
    f.write(proto_input_data.SerializeToString())
    f.close()


if __name__ == '__main__':
    if len(sys.argv) != 2:
        print("Usage: {} <path/to/configuration/file.yaml>".format(sys.argv[0]))
    else:
        run(sys.argv[1])



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import sys
import os
import re
import logging as log
from pathlib import Path

import pandas as pd

from famepy.source.tools import set_logfile
from famepy.protobuf_definitions import Services_pb2 as Service


def run(file_path, agents_to_extract=None):
    """Executes the main workflow for the reading of a FAME result file in protobuf format"""
    set_logfile("convertFameResults.log")
    source_file_name = file_path.split('\\')[-1]
    proto_buffer = read_proto_file(file_path)
    log.info('==> Create data frames and csv-files from results')

    for agent_type in proto_buffer.agentType:
        class_name = agent_type.className

        if agents_to_extract is not None:
            if class_name.upper() not in agents_to_extract:
                log.info('Output for agent {} ignored.'.format(class_name))
                continue
            else:
                agents_to_extract = agents_to_extract.replace(class_name.upper(), "")

        field_names_by_id = extract_agent_fields(agent_type)
        data_frames = create_data_frames(class_name, proto_buffer, field_names_by_id)

        output_folder_name = source_file_name.split(".")[1]
        if not os.path.exists(output_folder_name):
            os.mkdir(output_folder_name)

        out_file_name = output_folder_name + '/' + class_name + '.csv'
        if data_frames:
            write_to_file(out_file_name, data_frames)
        else:
            log.info('No output data for agent {}'.format(class_name))

    if agents_to_extract is not None:
        unknown_agents = re.sub("[,; \s]", " ", agents_to_extract)
        unknown_agents = re.sub("\s+", " ", unknown_agents)

        if len(unknown_agents) > 0:
            log.info('==> Unknown agents specified: {}'.format(unknown_agents))


def read_proto_file(proto_file_name):
    """Reads protobuf file with given `proto_file_name`"""
    log.info('==> Read data from file {}'.format(proto_file_name))
    proto_buffer = Service.Output()
    file = open(Path(proto_file_name).as_posix(), 'rb')
    proto_buffer.ParseFromString(file.read())
    file.close()
    return proto_buffer


def extract_agent_fields(agent_type):
    """Returns dictionary of column_names by column_id of given `agent_type`"""
    field_names_by_id = {}
    for field in agent_type.field:
        field_names_by_id[field.fieldId] = field.fieldName
    log.info('Added fields {} for agent {}'.format(field_names_by_id, agent_type.className))
    return field_names_by_id


def create_data_frames(class_name, proto_buffer, agent_fields):
    """Returns list of DataFrames each containing output (timestep, value) of one agent (by id) and column (by name)"""
    data_frames = []
    for series in proto_buffer.series:
        if class_name == series.className:
            column_data = extract_series(series, agent_fields)
            column_data_frames = build_frames_from_column_data(column_data)
            data_frames.extend(column_data_frames)
    return data_frames


def extract_series(series, agent_fields):
    """
    Returns a dictionary with the following shape:

    'times': dictionary mapping each output column (by name) to list of timesteps where an output occured
    'agentId': id of agent that created this output
    'values': dictionary mapping each output column (by name) to list of values for each timesteps with an output
    """
    values_dict = {}
    time_index_dict = {}

    for fieldId in agent_fields.keys():
        values_dict[fieldId] = []
        time_index_dict[fieldId] = []

    for line in series.line:
        current_time_step = line.timeStep
        for column in line.column:
            values_dict[column.fieldId].append(column.value)
            time_index_dict[column.fieldId].append(current_time_step)

    # rename fields from column_id to match column names
    values_dict = {agent_fields[k]: v for k, v in values_dict.items()}
    time_index_dict = {agent_fields[k]: v for k, v in time_index_dict.items()}

    return {'times': time_index_dict,
            'agentId': series.agentId,
            'values': values_dict}


def build_frames_from_column_data(column_data):
    """Returns list of DataFrames each entry resembling an output column with 'TimeStep' and 'AgentId' index"""
    column_data_frames = []
    values = column_data['values']
    times = column_data['times']
    for col_name, value_list in values.items():
        data_frame = pd.DataFrame({'TimeStep': times[col_name],
                                   'AgentId': column_data['agentId'],
                                   col_name: value_list})
        data_frame.set_index(['TimeStep', 'AgentId'], inplace=True)
        column_data_frames.append(data_frame)
    return column_data_frames


def write_to_file(out_file_name, data_frames):
    """Joins all DataFrames grouping by 'TimeStep' and 'AgentId' and exports result to disk"""
    joined_data_frame = pd.concat(data_frames, sort=False)
    grouped_joined_frame = joined_data_frame.groupby(['TimeStep', 'AgentId']).sum()
    grouped_joined_frame.sort_index(inplace=True)
    grouped_joined_frame.to_csv(out_file_name, sep=';', header=True, index=True)


if __name__ == '__main__':
    if len(sys.argv) > 2:
        run(sys.argv[1], sys.argv[2].upper())
    elif len(sys.argv) == 2:
        run(sys.argv[1])
    else:
        print("Usage: {} <path/to/result/file.pb> ['list', 'of', 'agents']".format(sys.argv[0]))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: InputFile.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from famepy.protobuf_definitions import Contract_pb2 as Contract__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='InputFile.proto',
  package='iobinary',
  syntax='proto2',
  serialized_options=b'\n\024de.dlr.fame.protobufB\005Input',
  serialized_pb=b'\n\x0fInputFile.proto\x12\x08iobinary\x1a\x0e\x43ontract.proto\"\xc0\x06\n\tInputData\x12\r\n\x05runId\x18\x01 \x02(\x03\x12*\n\x06\x63onfig\x18\x02 \x02(\x0b\x32\x1a.iobinary.InputData.Config\x12>\n\x10outputParameters\x18\x03 \x01(\x0b\x32$.iobinary.InputData.OutputParameters\x12\x35\n\ntimeSeries\x18\x04 \x03(\x0b\x32!.iobinary.InputData.TimeSeriesDao\x12+\n\x05\x61gent\x18\x05 \x03(\x0b\x32\x1c.iobinary.InputData.AgentDao\x12.\n\x08\x63ontract\x18\x06 \x03(\x0b\x32\x1c.communication.ProtoContract\x1a\x41\n\x06\x43onfig\x12\x11\n\tstartTime\x18\x01 \x02(\x03\x12\x10\n\x08stopTime\x18\x02 \x02(\x03\x12\x12\n\nrandomSeed\x18\x03 \x02(\x03\x1aZ\n\x10OutputParameters\x12\x16\n\x0eoutputInterval\x18\x01 \x02(\x05\x12\x15\n\routputProcess\x18\x02 \x02(\x05\x12\x17\n\x0f\x61\x63tiveClassName\x18\x03 \x03(\t\x1a\x91\x01\n\rTimeSeriesDao\x12\x10\n\x08seriesId\x18\x01 \x02(\x05\x12\x12\n\nseriesName\x18\x02 \x01(\t\x12\x32\n\x03row\x18\x03 \x03(\x0b\x32%.iobinary.InputData.TimeSeriesDao.Row\x1a&\n\x03Row\x12\x10\n\x08timeStep\x18\x01 \x02(\x03\x12\r\n\x05value\x18\x02 \x02(\x01\x1a\xf0\x01\n\x08\x41gentDao\x12\n\n\x02id\x18\x01 \x02(\x03\x12\x11\n\tclassName\x18\x02 \x02(\t\x12\x31\n\x05\x66ield\x18\x03 \x03(\x0b\x32\".iobinary.InputData.AgentDao.Field\x1a\x91\x01\n\x05\x46ield\x12\x11\n\tfieldName\x18\x01 \x02(\t\x12\x1d\n\x08seriesId\x18\x02 \x01(\x05:\x0b-2147483648\x12\x1d\n\x08intValue\x18\x03 \x01(\x05:\x0b-2147483648\x12\"\n\x0bstringValue\x18\x04 \x01(\t:\rENTRY_MISSING\x12\x13\n\x0b\x64oubleValue\x18\x05 \x03(\x01\x42\x1d\n\x14\x64\x65.dlr.fame.protobufB\x05Input'
  ,
  dependencies=[Contract__pb2.DESCRIPTOR,])




_INPUTDATA_CONFIG = _descriptor.Descriptor(
  name='Config',
  full_name='iobinary.InputData.Config',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='startTime', full_name='iobinary.InputData.Config.startTime', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stopTime', full_name='iobinary.InputData.Config.stopTime', index=1,
      number=2, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='randomSeed', full_name='iobinary.InputData.Config.randomSeed', index=2,
      number=3, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=330,
  serialized_end=395,
)

_INPUTDATA_OUTPUTPARAMETERS = _descriptor.Descriptor(
  name='OutputParameters',
  full_name='iobinary.InputData.OutputParameters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='outputInterval', full_name='iobinary.InputData.OutputParameters.outputInterval', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputProcess', full_name='iobinary.InputData.OutputParameters.outputProcess', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='activeClassName', full_name='iobinary.InputData.OutputParameters.activeClassName', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=397,
  serialized_end=487,
)

_INPUTDATA_TIMESERIESDAO_ROW = _descriptor.Descriptor(
  name='Row',
  full_name='iobinary.InputData.TimeSeriesDao.Row',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeStep', full_name='iobinary.InputData.TimeSeriesDao.Row.timeStep', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='iobinary.InputData.TimeSeriesDao.Row.value', index=1,
      number=2, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=597,
  serialized_end=635,
)

_INPUTDATA_TIMESERIESDAO = _descriptor.Descriptor(
  name='TimeSeriesDao',
  full_name='iobinary.InputData.TimeSeriesDao',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seriesId', full_name='iobinary.InputData.TimeSeriesDao.seriesId', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seriesName', full_name='iobinary.InputData.TimeSeriesDao.seriesName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='row', full_name='iobinary.InputData.TimeSeriesDao.row', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_INPUTDATA_TIMESERIESDAO_ROW, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=490,
  serialized_end=635,
)

_INPUTDATA_AGENTDAO_FIELD = _descriptor.Descriptor(
  name='Field',
  full_name='iobinary.InputData.AgentDao.Field',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fieldName', full_name='iobinary.InputData.AgentDao.Field.fieldName', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seriesId', full_name='iobinary.InputData.AgentDao.Field.seriesId', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=-2147483648,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='intValue', full_name='iobinary.InputData.AgentDao.Field.intValue', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=-2147483648,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stringValue', full_name='iobinary.InputData.AgentDao.Field.stringValue', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=True, default_value=b"ENTRY_MISSING".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='doubleValue', full_name='iobinary.InputData.AgentDao.Field.doubleValue', index=4,
      number=5, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=878,
)

_INPUTDATA_AGENTDAO = _descriptor.Descriptor(
  name='AgentDao',
  full_name='iobinary.InputData.AgentDao',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='iobinary.InputData.AgentDao.id', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='className', full_name='iobinary.InputData.AgentDao.className', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='field', full_name='iobinary.InputData.AgentDao.field', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_INPUTDATA_AGENTDAO_FIELD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=638,
  serialized_end=878,
)

_INPUTDATA = _descriptor.Descriptor(
  name='InputData',
  full_name='iobinary.InputData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='runId', full_name='iobinary.InputData.runId', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='iobinary.InputData.config', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='outputParameters', full_name='iobinary.InputData.outputParameters', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeSeries', full_name='iobinary.InputData.timeSeries', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='agent', full_name='iobinary.InputData.agent', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contract', full_name='iobinary.InputData.contract', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_INPUTDATA_CONFIG, _INPUTDATA_OUTPUTPARAMETERS, _INPUTDATA_TIMESERIESDAO, _INPUTDATA_AGENTDAO, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=878,
)

_INPUTDATA_CONFIG.containing_type = _INPUTDATA
_INPUTDATA_OUTPUTPARAMETERS.containing_type = _INPUTDATA
_INPUTDATA_TIMESERIESDAO_ROW.containing_type = _INPUTDATA_TIMESERIESDAO
_INPUTDATA_TIMESERIESDAO.fields_by_name['row'].message_type = _INPUTDATA_TIMESERIESDAO_ROW
_INPUTDATA_TIMESERIESDAO.containing_type = _INPUTDATA
_INPUTDATA_AGENTDAO_FIELD.containing_type = _INPUTDATA_AGENTDAO
_INPUTDATA_AGENTDAO.fields_by_name['field'].message_type = _INPUTDATA_AGENTDAO_FIELD
_INPUTDATA_AGENTDAO.containing_type = _INPUTDATA
_INPUTDATA.fields_by_name['config'].message_type = _INPUTDATA_CONFIG
_INPUTDATA.fields_by_name['outputParameters'].message_type = _INPUTDATA_OUTPUTPARAMETERS
_INPUTDATA.fields_by_name['timeSeries'].message_type = _INPUTDATA_TIMESERIESDAO
_INPUTDATA.fields_by_name['agent'].message_type = _INPUTDATA_AGENTDAO
_INPUTDATA.fields_by_name['contract'].message_type = Contract__pb2._PROTOCONTRACT
DESCRIPTOR.message_types_by_name['InputData'] = _INPUTDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

InputData = _reflection.GeneratedProtocolMessageType('InputData', (_message.Message,), {

  'Config' : _reflection.GeneratedProtocolMessageType('Config', (_message.Message,), {
    'DESCRIPTOR' : _INPUTDATA_CONFIG,
    '__module__' : 'InputFile_pb2'
    # @@protoc_insertion_point(class_scope:iobinary.InputData.Config)
    })
  ,

  'OutputParameters' : _reflection.GeneratedProtocolMessageType('OutputParameters', (_message.Message,), {
    'DESCRIPTOR' : _INPUTDATA_OUTPUTPARAMETERS,
    '__module__' : 'InputFile_pb2'
    # @@protoc_insertion_point(class_scope:iobinary.InputData.OutputParameters)
    })
  ,

  'TimeSeriesDao' : _reflection.GeneratedProtocolMessageType('TimeSeriesDao', (_message.Message,), {

    'Row' : _reflection.GeneratedProtocolMessageType('Row', (_message.Message,), {
      'DESCRIPTOR' : _INPUTDATA_TIMESERIESDAO_ROW,
      '__module__' : 'InputFile_pb2'
      # @@protoc_insertion_point(class_scope:iobinary.InputData.TimeSeriesDao.Row)
      })
    ,
    'DESCRIPTOR' : _INPUTDATA_TIMESERIESDAO,
    '__module__' : 'InputFile_pb2'
    # @@protoc_insertion_point(class_scope:iobinary.InputData.TimeSeriesDao)
    })
  ,

  'AgentDao' : _reflection.GeneratedProtocolMessageType('AgentDao', (_message.Message,), {

    'Field' : _reflection.GeneratedProtocolMessageType('Field', (_message.Message,), {
      'DESCRIPTOR' : _INPUTDATA_AGENTDAO_FIELD,
      '__module__' : 'InputFile_pb2'
      # @@protoc_insertion_point(class_scope:iobinary.InputData.AgentDao.Field)
      })
    ,
    'DESCRIPTOR' : _INPUTDATA_AGENTDAO,
    '__module__' : 'InputFile_pb2'
    # @@protoc_insertion_point(class_scope:iobinary.InputData.AgentDao)
    })
  ,
  'DESCRIPTOR' : _INPUTDATA,
  '__module__' : 'InputFile_pb2'
  # @@protoc_insertion_point(class_scope:iobinary.InputData)
  })
_sym_db.RegisterMessage(InputData)
_sym_db.RegisterMessage(InputData.Config)
_sym_db.RegisterMessage(InputData.OutputParameters)
_sym_db.RegisterMessage(InputData.TimeSeriesDao)
_sym_db.RegisterMessage(InputData.TimeSeriesDao.Row)
_sym_db.RegisterMessage(InputData.AgentDao)
_sym_db.RegisterMessage(InputData.AgentDao.Field)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import datetime as dt
import math


START_IN_REAL_TIME = '2000-01-01_00:00:00'
DATE_FORMAT = '%Y-%m-%d_%H:%M:%S'
FAME_FIRST_DATETIME = dt.datetime.strptime(START_IN_REAL_TIME, DATE_FORMAT)

# Constants
STEPS_PER_SECOND = 1
SECONDS_PER_MINUTE = 60
MINUTES_PER_HOUR = 60
HOURS_PER_DAY = 24
DAYS_PER_YEAR = 365
STEPS_PER_MINUTE = STEPS_PER_SECOND * SECONDS_PER_MINUTE
STEPS_PER_HOUR = STEPS_PER_MINUTE * MINUTES_PER_HOUR
STEPS_PER_DAY = STEPS_PER_HOUR * HOURS_PER_DAY
STEPS_PER_YEAR = STEPS_PER_DAY * DAYS_PER_YEAR


class FameTime:
    """Handles conversion of TimeSteps into TimeStamps and vice versa"""

    @staticmethod
    def convert_time_string_to_fame_time_step(datetime_string):
        """Converts real Datetime string to FAME time step"""
        datetime = dt.datetime.strptime(datetime_string, DATE_FORMAT)
        years_since_start_time = datetime.year - FAME_FIRST_DATETIME.year
        beginning_of_year = dt.datetime(year=datetime.year, month=1, day=1, hour=0, minute=0, second=0)
        seconds_since_beginning_of_year = (datetime - beginning_of_year).total_seconds()
        return years_since_start_time * STEPS_PER_YEAR + seconds_since_beginning_of_year

    @staticmethod
    def convert_fame_time_step_to_datetime_string(fame_time_steps):
        """Converts fame time step to Datetime string"""
        years_since_start_time = math.floor(fame_time_steps / STEPS_PER_YEAR)
        current_year = years_since_start_time + 2000
        beginning_of_year = dt.datetime(year=current_year, month=1, day=1, hour=0, minute=0, second=0)
        seconds_in_current_year = fame_time_steps - years_since_start_time * STEPS_PER_YEAR
        datetime = beginning_of_year + dt.timedelta(seconds=seconds_in_current_year)
        return datetime.strftime(DATE_FORMAT)



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import logging as log
import pathlib as pt

import yaml


def load_yaml(yaml_file_path):
    """Loads the yaml file from given `yaml_file_path` and returns its content"""
    with open(pt.Path(yaml_file_path), "r") as configfile:
        data = yaml.load(configfile,  Loader=yaml.FullLoader)
    return data


def log_and_raise(message):
    """ Raises a critical error and logs with given `error_message` """
    log.critical(message)
    raise Exception(message)


def set_logfile(file_name):
    """Open a log file with the given name"""
    log.basicConfig(filename=file_name,
                    filemode='w',
                    format='%(levelname)s:%(message)s',
                    level=log.DEBUG,
                    )